/*
 * Created on May 3, 2004
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Generation - Code and Comments
 */
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;

/**
 * @author tommut
 */
public class AlbumExport {

	private class AlbumInfo {
		public AlbumInfo(String artist, String album, String year, String genre) {
			super();
			this.artist = artist;
			this.album = album;
			this.year = year;
			this.genre = genre;
		}
		String artist;
		String album;
		String year;
		String genre;
	}
	
	public static Comparator<AlbumInfo> albumComparator = new Comparator<AlbumInfo>() {
		public int compare(AlbumInfo albumInfo1, AlbumInfo albumInfo2) {
			String artist1 = albumInfo1.artist;
			String artist2 = albumInfo2.artist;
			// ignore "The" when sorting
			if ( artist1.startsWith("The ") ) {
				artist1 = artist1.substring( "The ".length());							
			}
			if ( artist2.startsWith("The ") ) {
				artist2 = artist2.substring( "The ".length());							
			}
			
			// ignore " around names (eg: "Weird Al" Yankovich)
			if ( artist1.startsWith("\"") ) {
				artist1 = artist1.substring( "\"".length());							
			}
			if ( artist2.startsWith("\"") ) {
				artist2 = artist2.substring( "\"".length());							
			}
			
			
			String title1 = albumInfo1.album;
			String title2 = albumInfo2.album;
			if ( title1.startsWith("The ") ) {
				title1 = title1.substring( "The ".length());							
			}
			if ( title2.startsWith("The ") ) {
				title2 = title2.substring( "The ".length());							
			}
			
			int compareTo = artist1.compareTo( artist2);
			if ( compareTo == 0 ) {
				compareTo = title1.compareTo( title2 );
			}
			return compareTo;
		}
	};
	
	private List<AlbumInfo> albumList = new ArrayList<AlbumInfo>();
	
	
	public static void main(String[] args) {
		AlbumExport export = new AlbumExport();
		String fileName = null;
		if ( args.length > 0 ) {
			fileName = args[0];
		}
		else {
			System.out.println( "usage: java AlbumExport musicLibraryFileName.xml");
		}
		if ( fileName == null ) {
			fileName = "D:/share/Library.xml";
		}
		export.exportAlbums(fileName);
	}
	
	public void exportAlbums( String fileName ) {
		getAlbums( fileName );
		export2Html( fileName );
	}
	
	
	private void getAlbums( String fileName ) {
		HashMap<String,AlbumInfo> albumInfos = new HashMap<String,AlbumInfo>();
		BufferedReader reader = null;
		try {
			Reader in = new InputStreamReader(new FileInputStream(fileName), "UTF-8");

			//reader = new BufferedReader(new FileReader(fileName));
			reader = new BufferedReader(in);
			String line = null;
			String artist = null;
			String album = null;
			String year = null;
			String genre = null;
			boolean podcast = false;
			boolean video = false;
			while((line = reader.readLine()) != null)	{
				if ( line.indexOf( "<dict>" )!= -1 ) {
					artist = null;
					album = null;
					year = null;					
				}
				else if ( line.indexOf("<key>Artist</key>") != -1 ) {
					int start = line.indexOf("<string>"	);
					int end = line.indexOf("</string>");
								
					artist = line.substring( start+"<string>".length(), end);
				}
				else if ( line.indexOf("<key>Album</key>") != -1 ) {
					int start = line.indexOf("<string>"	);
					int end = line.indexOf("</string>");
								
					album = line.substring( start+"<string>".length(), end);
				}
				else if ( line.indexOf("<key>Podcast</key>") != -1 ) {
					if ( line.contains("<true/>" ) ) {
						podcast = true;
					}
				}
				else if ( line.indexOf("<key>Podcast</key>") != -1 ) {
					if ( line.contains("<true/>" ) ) {
						podcast = true;
					}
				}
				else if ( line.indexOf("<key>Has Video</key>") != -1 ) {
					if ( line.contains("<true/>" ) ) {
						video = true;
					}
				}
				else if ( line.indexOf("<key>Genre</key>") != -1 ) {
					int start = line.indexOf("<string>"	);
					int end = line.indexOf("</string>");
								
					genre = line.substring( start+"<string>".length(), end);
				}
				else if ( line.indexOf("<key>Year</key>") != -1 ) {
					int start = line.indexOf("<integer>"	);
					int end = line.indexOf("</integer>");
								
					year = line.substring( start+"<integer>".length(), end);
				}
				else if ( line.indexOf( "</dict>" )!= -1 ) { 
					if ( artist != null && album != null ) {	
						if ( video || podcast || album.equalsIgnoreCase("podcast" ) || album.equalsIgnoreCase("audiobook") 
								|| (genre != null && (genre.equalsIgnoreCase("podcast") || genre.equalsIgnoreCase("audiobook")) ) ) {
							// ignore
						}
						else {
							String albumKey = artist + "***" + album;
							
							if ( !albumInfos.containsKey(albumKey)) {
								albumInfos.put(albumKey, new AlbumInfo( artist, album, year, genre));
							}
						}
						artist = null;
						album = null;
						year = null;
						genre = null;
						podcast = false;
						video = false;
					}

					
				}
			}
			
			
			for (AlbumInfo info : albumInfos.values()) {
				albumList.add(info);
			}
			Collections.sort(albumList, albumComparator );
				
				
		} catch(FileNotFoundException fe) {
           fe.printStackTrace();
		} catch(IOException ie) {
            ie.printStackTrace();
		} finally {
			if (reader != null) {
				try {
					reader.close();
				} catch( IOException ie2 ) {
		            ie2.printStackTrace();
				}
			}
		}
	}
	
	private void export2Html( String fileName ) {
		BufferedWriter writer = null;
		StringBuffer buffer = new StringBuffer();
		
		buffer.append("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 // Transitional//EN\">\n");
		buffer.append("<HTML>\n");
		buffer.append("<HEAD>\n");
		buffer.append("<META http-equiv=\"Content-Type\" content=\"text/html; charset=ISO-8859-1\">\n");
		buffer.append("<TITLE>iTunes Albums</TITLE>\n");
		buffer.append("</HEAD>\n");
		buffer.append("<BODY>\n");
		buffer.append("<h3>iTunes Albums</h3>\n");
		buffer.append("<TABLE border=\"1\">\n");
		buffer.append(" <TBODY>\n");
		
		
		try {
			writer = new BufferedWriter(new FileWriter(fileName + ".html"));
			String line = null;
			buffer.append( "  <TR>\n");
			
			buffer.append( "    <TD>\n");
			buffer.append( "      <b>#</b>\n");
			buffer.append( "    </TD>\n");
			
			buffer.append( "    <TD>\n");
			buffer.append( "      <b>Artist</b>\n");
			buffer.append( "    </TD>\n");
			
			buffer.append( "    <TD>\n");
			buffer.append( "      <b>Album</b>\n" );
			buffer.append( "    </TD>\n");
			
			buffer.append( "    <TD>\n");
			buffer.append( "      <b>Year&nbsp;&nbsp;</b>\n");
			buffer.append( "    </TD>\n");
			
			buffer.append( "    <TD>\n");
			buffer.append( "      <b>Genre</b>\n");
			buffer.append( "    </TD>\n");
			
			buffer.append( "  </TR>\n");
			
			for (int i = 0; i < albumList.size(); i++ ) {
				AlbumInfo albumInfo = albumList.get(i);
				buffer.append( "  <TR>\n");
				
				buffer.append( "    <TD>\n");
				buffer.append( "      ");
				buffer.append( i +1 );
				buffer.append( '\n');
				buffer.append( "    </TD>\n");
				
				buffer.append( "    <TD>\n      ");
				buffer.append( albumInfo.artist + '\n');
				buffer.append( "    </TD>\n");
				
				buffer.append( "    <TD>\n      ");
				buffer.append( albumInfo.album + '\n' );
				buffer.append( "    </TD>\n");
				
				buffer.append( "    <TD>\n      ");
				if ( albumInfo.year != null ) {
					buffer.append( albumInfo.year + '\n');
				}
				else {
					buffer.append( "&nbsp;\n");
				}
				buffer.append( "    </TD>\n");
				
				buffer.append( "    <TD>\n      ");
				if ( albumInfo.genre != null ) {
					buffer.append( albumInfo.genre + '\n');
				}
				else {
					buffer.append( "&nbsp;\n");
				}
				buffer.append( "    </TD>\n");
				
				buffer.append( "  </TR>\n");
			}
			
			buffer.append( " </TBODY>\n");
			buffer.append( "</TABLE>\n");
			buffer.append( "<I><small>Generated " + Calendar.getInstance().getTime()+ "</small></I><BR/>\n");
			buffer.append( "</BODY>\n");
			buffer.append( "</HTML>\n");
			
			writer.write( buffer.toString());
			
			System.out.println( "Exported library to: " + fileName +".html");
				
		} catch(FileNotFoundException fe) {
           fe.printStackTrace();
		} catch(IOException ie) {
            ie.printStackTrace();
		} finally {
			if (writer != null) {
				try {
					writer.close();
				} catch( IOException ie2 ) {
		            ie2.printStackTrace();
				}
			}
		}
	}
	
}
